#!/bin/sh
#
# Demonstration on how to use Glade and the GTK-server with BASH.
#
# Tested with GTK-server 2.1.3 on Zenwalk 4.2 and BASH 3.1
# January 16, 2007 (c) Peter van Eerten - GPL license.
#
#----------------------------------------------------------- Setup embedded GTK

# Name of PIPE file
export PIPE=/tmp/gtk-server-fifo.$$

# create it so we don't need to busy wait for it (needs gtk-server hack)
mkfifo $PIPE

cd `dirname $0`

# Start gtk-server
./gtk-server fifo=$PIPE log &

GTK=""
NULL="NULL"

# Include the generated file to use embedded GTK functions
. ./.gtk4sh

#---------------------------------------------------------- Global inits

# GTK-server must go after an error in the script
#trap 'kill-gtk-server fifo >/dev/null 2>&1' ERR

# Define SIGUSR1 in GTK-server configfile to catch exit of GTK-server
trap 'exit' SIGUSR1

# Setup environment
#export LC_ALL=nl_NL
#export LD_LIBRARY_PATH=/usr/X11R6/lib

#---------------------------------------------------------- GUI realization

# Get Glade file and realize GUI
gtk_server_glade_file "calculator.glade"

# Get main window ID and connect signal
gtk_server_glade_widget "window"
gtk_server_connect "$GTK delete-event window"

# Get button ID's and connect signals
gtk_server_glade_widget "button0"
gtk_server_connect "$GTK clicked button0"
gtk_server_glade_widget "button1"
gtk_server_connect "$GTK clicked button1"
gtk_server_glade_widget "button2"
gtk_server_connect "$GTK clicked button2"
gtk_server_glade_widget "button3"
gtk_server_connect "$GTK clicked button3"
gtk_server_glade_widget "button4"
gtk_server_connect "$GTK clicked button4"
gtk_server_glade_widget "button5"
gtk_server_connect "$GTK clicked button5"
gtk_server_glade_widget "button6"
gtk_server_connect "$GTK clicked button6"
gtk_server_glade_widget "button7"
gtk_server_connect "$GTK clicked button7"
gtk_server_glade_widget "button8"
gtk_server_connect "$GTK clicked button8"
gtk_server_glade_widget "button9"
gtk_server_connect "$GTK clicked button9"
# Operators
gtk_server_glade_widget "buttonC"
gtk_server_connect "$GTK clicked buttonC"
gtk_server_glade_widget "buttonCE"
gtk_server_connect "$GTK clicked buttonCE"
gtk_server_glade_widget "buttonAdd"
gtk_server_connect "$GTK clicked buttonAdd"
gtk_server_glade_widget "buttonMinus"
gtk_server_connect "$GTK clicked buttonMinus"
gtk_server_glade_widget "buttonMul"
gtk_server_connect "$GTK clicked buttonMul"
gtk_server_glade_widget "buttonDiv"
gtk_server_connect "$GTK clicked buttonDiv"
gtk_server_glade_widget "buttonEq"
gtk_server_connect "$GTK clicked buttonEq"
# Memory buttons
gtk_server_glade_widget "buttonMemadd"
gtk_server_connect "$GTK clicked buttonMemadd"
gtk_server_glade_widget "buttonMemread"
gtk_server_connect "$GTK clicked buttonMemread"
# Get entry ID
gtk_server_glade_widget "entry"
ENTRY=$GTK
# Calculator starts with 0
gtk_entry_set_text $ENTRY 0

#---------------------------------------------------------- Init variables

# Initialize calculator state
STATE=0
# Initialize temp calculator cache
CACHE=0
# Initialize last calculator action
ACTION=0
# Initiliaze calculator total result
TOTAL=0
# Initialize MEM function
MEM=0

#---------------------------------------------------------- Functions

# Define action when +, -, *, or / is pressed
operator()
{
if [ $STATE -eq 0 ]
then
    case $ACTION in
	1)
	    gtk_entry_get_text $ENTRY
	    let CACHE=$CACHE+$GTK
	    gtk_entry_set_text $ENTRY $CACHE;;
	2)
	    gtk_entry_get_text $ENTRY
	    let CACHE=$CACHE-$GTK
	    gtk_entry_set_text $ENTRY $CACHE;;
	3)
	    gtk_entry_get_text $ENTRY
	    let CACHE=$CACHE*$GTK
	    gtk_entry_set_text $ENTRY $CACHE;;
	4)
	    gtk_entry_get_text $ENTRY
	    let CACHE=$CACHE/$GTK
	    gtk_entry_set_text $ENTRY $CACHE;;
    esac
fi
}

# Define action when 0 1 2 3 4 5 6 7 8 9 is pressed; requires argument
number()
{
gtk_entry_get_text $ENTRY
if [ $GTK -eq 0 -o $STATE -gt 0 ]
then
    gtk_entry_get_text $ENTRY
    let CACHE=$GTK
    gtk_entry_set_text $ENTRY $1
else
    gtk_entry_set_text $ENTRY $GTK$1
fi
STATE=0
}

#---------------------------------------------------------- Mainloop
EVENT=""

while [ ! "$EVENT" = "window" ]
do
    gtk_server_callback "wait"
    EVENT=$GTK

    case "$EVENT" in
	"buttonMemread")
	    STATE=0
	    gtk_entry_get_text $ENTRY
	    CACHE=$GTK
	    gtk_entry_set_text $ENTRY $MEM;;
	"buttonMemadd")
	    gtk_entry_get_text $ENTRY
	    MEM=$GTK;;
	"buttonC")
	    gtk_entry_set_text $ENTRY 0;;
	"buttonCE")
	    TOTAL=0
	    STATE=0
	    ACTION=0
	    CACHE=0
	    MEM=0
	    gtk_entry_set_text $ENTRY 0;;
	"buttonAdd")
	    operator
	    ACTION=1
	    STATE=$((STATE+1));;
	"buttonMinus")
	    operator
	    ACTION=2
	    STATE=$((STATE+1));;
	"buttonMul")
	    operator
	    ACTION=3
	    STATE=$((STATE+1));;
	"buttonDiv")
	    operator
	    ACTION=4
	    STATE=$((STATE+1));;
	"buttonEq")
	    operator
	    ACTION=0
	    STATE=$((STATE+1));;
	"button0")
	    number 0;;
	"button1")
	    number 1;;
	"button2")
	    number 2;;
	"button3")
	    number 3;;
	"button4")
	    number 4;;
	"button5")
	    number 5;;
	"button6")
	    number 6;;
	"button7")
	    number 7;;
	"button8")
	    number 8;;
	"button9")
	    number 9;;
    esac
done

# Exit GTK
gtk_exit 0
